"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Bluebird = require("bluebird");
const Express = require("express");
const ConfigurationParser_1 = require("./configuration/ConfigurationParser");
const GlobalLogger_1 = require("./logging/GlobalLogger");
const RequestLogger_1 = require("./logging/RequestLogger");
const ServerVariablesInitializer_1 = require("./ServerVariablesInitializer");
const Configurator_1 = require("./web_server/Configurator");
const constants_1 = require("./constants");
function clone(obj) {
    return JSON.parse(JSON.stringify(obj));
}
class Server {
    constructor(deps) {
        this.globalLogger = new GlobalLogger_1.GlobalLogger(deps.winston);
        this.requestLogger = new RequestLogger_1.RequestLogger(deps.winston);
    }
    displayConfigurations(configuration) {
        const displayableConfiguration = clone(configuration);
        const STARS = "*****";
        if (displayableConfiguration.authentication_backend.ldap) {
            displayableConfiguration.authentication_backend.ldap.password = STARS;
        }
        displayableConfiguration.session.secret = STARS;
        if (displayableConfiguration.notifier && displayableConfiguration.notifier.email)
            displayableConfiguration.notifier.email.password = STARS;
        if (displayableConfiguration.notifier && displayableConfiguration.notifier.smtp)
            displayableConfiguration.notifier.smtp.password = STARS;
        if (displayableConfiguration.duo_api) {
            displayableConfiguration.duo_api.secret_key = STARS;
        }
        this.globalLogger.debug("User configuration is %s", JSON.stringify(displayableConfiguration, undefined, 2));
    }
    setup(config, app, deps) {
        return ServerVariablesInitializer_1.ServerVariablesInitializer.initialize(config, this.globalLogger, this.requestLogger, deps)
            .then(function (vars) {
            app.set(constants_1.GET_VARIABLE_KEY, vars);
            return Configurator_1.Configurator.configure(config, app, vars, deps);
        });
    }
    startServer(app, port) {
        const that = this;
        that.globalLogger.info("Starting Authelia...");
        return new Bluebird((resolve, reject) => {
            this.httpServer = app.listen(port, function (err) {
                that.globalLogger.info("Listening on port %d...", port);
                resolve();
            });
        });
    }
    start(configuration, deps) {
        const that = this;
        const app = Express();
        const appConfiguration = ConfigurationParser_1.ConfigurationParser.parse(configuration);
        deps.winston.level = appConfiguration.logs_level;
        if (process.env.LDAP_BACKEND_PASSWORD) {
            if (appConfiguration.authentication_backend.ldap) {
                appConfiguration.authentication_backend.ldap.password = process.env.LDAP_BACKEND_PASSWORD;
                that.globalLogger.debug("Got ldap binding password from environment");
            }
            else {
                const erMsg = "Environment variable LDAP_BACKEND_PASSWORD set, but no ldap configuration is specified in configuration file.";
                that.globalLogger.error(erMsg);
                throw new Error(erMsg);
            }
        }
        if (process.env.SESSION_SECRET) {
            appConfiguration.session.secret = process.env.SESSION_SECRET;
            that.globalLogger.debug("Got session secret from environment");
        }
        if (process.env.EMAIL_SERVICE_PASSWORD) {
            if (appConfiguration.notifier && appConfiguration.notifier.email) {
                appConfiguration.notifier.email.password = process.env.EMAIL_SERVICE_PASSWORD;
                that.globalLogger.debug("Got e-mail service notifier password from environment");
            }
            else {
                const erMsg = "Environment variable EMAIL_SERVICE_PASSWORD set, but no e-mail service is given in the " +
                    "notifier section of the configuration file.";
                that.globalLogger.error(erMsg);
                throw new Error(erMsg);
            }
        }
        if (process.env.SMTP_PASSWORD) {
            if (appConfiguration.notifier && appConfiguration.notifier.smtp) {
                appConfiguration.notifier.smtp.password = process.env.SMTP_PASSWORD;
                that.globalLogger.debug("Got smtp service notifier password from environment");
            }
            else {
                const erMsg = "Environment variable SMTP_PASSWORD set, but no smtp entry is given in the notifier section of " +
                    "the configuration file.";
                that.globalLogger.error(erMsg);
                throw new Error(erMsg);
            }
        }
        if (process.env.DUO_API_SECRET_KEY) {
            if (appConfiguration.duo_api) {
                appConfiguration.duo_api.secret_key = process.env.DUO_API_SECRET_KEY;
                that.globalLogger.debug("Got duo api secret from environment");
            }
            else {
                const erMsg = "Environment variable DUO_API_SECRET_KEY set, but no duo_api section given in the configuration file.";
                that.globalLogger.error(erMsg);
                throw new Error(erMsg);
            }
        }
        this.displayConfigurations(appConfiguration);
        return this.setup(appConfiguration, app, deps)
            .then(function () {
            return that.startServer(app, appConfiguration.port);
        });
    }
    stop() {
        this.httpServer.close();
    }
}
exports.default = Server;
//# sourceMappingURL=Server.js.map